@@ -118,11 +118,11 @@ public class Constants {
             "<table cellspacing=\"4\" border=\"0\">\n" +
             " <tr>\n" +
             "  <td colspan=\"2\">\n" +
-            "   <a href=\"http://tomcat.apache.org/\">\n" +
+            "   <a href=\"https://tomcat.apache.org/\">\n" +
             "    <img border=\"0\" alt=\"The Tomcat Servlet/JSP Container\"\n" +
             "         align=\"left\" src=\"{0}/images/tomcat.gif\">\n" +
             "   </a>\n" +
-            "   <a href=\"http://www.apache.org/\">\n" +
+            "   <a href=\"https://www.apache.org/\">\n" +
             "    <img border=\"0\" alt=\"The Apache Software Foundation\" align=\"right\"\n" +
             "         src=\"{0}/images/asf-logo.svg\" style=\"width: 266px; height: 83px;\">\n" +
             "   </a>\n" +
