@@ -61,7 +61,9 @@ public class UserDatabaseRealm
 
     /**
      * Descriptive information about this Realm implementation.
+     * @deprecated This will be removed in Tomcat 9 onwards.
      */
+    @Deprecated
     protected static final String name = "UserDatabaseRealm";
 
 
@@ -155,14 +157,10 @@ public class UserDatabaseRealm
     // ------------------------------------------------------ Protected Methods
 
 
-    /**
-     * Return a short name for this Realm implementation.
-     */
     @Override
+    @Deprecated
     protected String getName() {
-
-        return (name);
-
+        return name;
     }
 
 
