@@ -121,6 +121,12 @@ public class Catalina {
     protected boolean useNaming = true;
 
 
+    /**
+     * Prevent duplicate loads.
+     */
+    protected boolean loaded = false;
+
+
     // ----------------------------------------------------------- Constructors
 
     public Catalina() {
@@ -517,6 +523,11 @@ public class Catalina {
      */
     public void load() {
 
+        if (loaded) {
+            return;
+        }
+        loaded = true;
+
         long t1 = System.nanoTime();
 
         initDirs();
