@@ -50,8 +50,7 @@ import org.apache.juli.logging.LogFactory;
 public class NioSender extends AbstractSender {
 
     private static final Log log = LogFactory.getLog(NioSender.class);
-    protected static final StringManager sm =
-            StringManager.getManager(NioSender.class.getPackage().getName());
+    protected static final StringManager sm = StringManager.getManager(NioSender.class);
 
 
     protected Selector selector;
@@ -76,10 +75,11 @@ public class NioSender extends AbstractSender {
     }
 
     /**
-     * State machine to send data
-     * @param key SelectionKey
-     * @return boolean
-     * @throws IOException
+     * State machine to send data.
+     * @param key The key to use
+     * @param waitForAck Wait for an ack
+     * @return <code>true</code> if the processing was successful
+     * @throws IOException An IO error occurred
      */
     public boolean process(SelectionKey key, boolean waitForAck) throws IOException {
         int ops = key.readyOps();
@@ -126,8 +126,8 @@ public class NioSender extends AbstractSender {
             }//end if
         } else {
             //unknown state, should never happen
-            log.warn(sm.getString("nioSender.unknown.state", ops));
-            throw new IOException(sm.getString("nioSender.unknown.state", ops));
+            log.warn(sm.getString("nioSender.unknown.state", Integer.toString(ops)));
+            throw new IOException(sm.getString("nioSender.unknown.state", Integer.toString(ops)));
         }//end if
         return false;
     }
