@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -17,8 +17,9 @@
 
 package org.apache.catalina.util;
 
-import java.util.Hashtable;
+import java.util.HashMap;
 import java.util.Locale;
+import java.util.Map;
 
 /**
  * MIME2Java is a convenience class which handles conversions between MIME charset names
@@ -29,453 +30,456 @@ import java.util.Locale;
  * as <code>TXDocument#setEncoding</code> and <code>DTD#setEncoding</code>.
  * <p>Java encoding names are used on <var>encoding</var> parameters to
  * methods such as <code>TXDocument#printWithFormat</code> and <code>DTD#printExternal</code>.
- * <P>
  * <TABLE BORDER="0" WIDTH="100%">
+ *  <caption>MIME charset name to Java encoding name mapping</caption>
  *  <TR>
- *      <TD WIDTH="33%">
- *          <P ALIGN="CENTER"><B>Common Name</B>
+ *      <TD>
+ *          <P><B>Common Name</B>
  *      </TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER"><B>Use this name in XML files</B>
+ *      <TD>
+ *          <P><B>Use this name in XML files</B>
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER"><B>Name Type</B>
+ *      <TD>
+ *          <P><B>Name Type</B>
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER"><B>Xerces converts to this Java Encoder Name</B>
+ *      <TD>
+ *          <P><B>Xerces converts to this Java Encoder Name</B>
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">8 bit Unicode</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">UTF-8
+ *      <TD>8 bit Unicode</TD>
+ *      <TD>
+ *          <P>UTF-8
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">UTF8
+ *      <TD>
+ *          <P>UTF8
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">ISO Latin 1</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ISO-8859-1
+ *      <TD>ISO Latin 1</TD>
+ *      <TD>
+ *          <P>ISO-8859-1
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">MIME
+ *      <TD>
+ *          <P>MIME
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">ISO-8859-1
+ *      <TD>
+ *          <P>ISO-8859-1
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">ISO Latin 2</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ISO-8859-2
+ *      <TD>ISO Latin 2</TD>
+ *      <TD>
+ *          <P>ISO-8859-2
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">MIME
+ *      <TD>
+ *          <P>MIME
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">ISO-8859-2
+ *      <TD>
+ *          <P>ISO-8859-2
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">ISO Latin 3</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ISO-8859-3
+ *      <TD>ISO Latin 3</TD>
+ *      <TD>
+ *          <P>ISO-8859-3
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">MIME
+ *      <TD>
+ *          <P>MIME
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">ISO-8859-3
+ *      <TD>
+ *          <P>ISO-8859-3
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">ISO Latin 4</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ISO-8859-4
+ *      <TD>ISO Latin 4</TD>
+ *      <TD>
+ *          <P>ISO-8859-4
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">MIME
+ *      <TD>
+ *          <P>MIME
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">ISO-8859-4
+ *      <TD>
+ *          <P>ISO-8859-4
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">ISO Latin Cyrillic</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ISO-8859-5
+ *      <TD>ISO Latin Cyrillic</TD>
+ *      <TD>
+ *          <P>ISO-8859-5
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">MIME
+ *      <TD>
+ *          <P>MIME
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">ISO-8859-5
+ *      <TD>
+ *          <P>ISO-8859-5
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">ISO Latin Arabic</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ISO-8859-6
+ *      <TD>ISO Latin Arabic</TD>
+ *      <TD>
+ *          <P>ISO-8859-6
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">MIME
+ *      <TD>
+ *          <P>MIME
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">ISO-8859-6
+ *      <TD>
+ *          <P>ISO-8859-6
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">ISO Latin Greek</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ISO-8859-7
+ *      <TD>ISO Latin Greek</TD>
+ *      <TD>
+ *          <P>ISO-8859-7
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">MIME
+ *      <TD>
+ *          <P>MIME
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">ISO-8859-7
+ *      <TD>
+ *          <P>ISO-8859-7
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">ISO Latin Hebrew</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ISO-8859-8
+ *      <TD>ISO Latin Hebrew</TD>
+ *      <TD>
+ *          <P>ISO-8859-8
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">MIME
+ *      <TD>
+ *          <P>MIME
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">ISO-8859-8
+ *      <TD>
+ *          <P>ISO-8859-8
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">ISO Latin 5</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ISO-8859-9
+ *      <TD>ISO Latin 5</TD>
+ *      <TD>
+ *          <P>ISO-8859-9
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">MIME
+ *      <TD>
+ *          <P>MIME
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">ISO-8859-9
+ *      <TD>
+ *          <P>ISO-8859-9
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: US</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-us
+ *      <TD>EBCDIC: US</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-us
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp037
+ *      <TD>
+ *          <P>cp037
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: Canada</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-ca
+ *      <TD>EBCDIC: Canada</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-ca
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp037
+ *      <TD>
+ *          <P>cp037
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: Netherlands</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-nl
+ *      <TD>EBCDIC: Netherlands</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-nl
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp037
+ *      <TD>
+ *          <P>cp037
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: Denmark</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-dk
+ *      <TD>EBCDIC: Denmark</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-dk
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp277
+ *      <TD>
+ *          <P>cp277
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: Norway</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-no
+ *      <TD>EBCDIC: Norway</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-no
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp277
+ *      <TD>
+ *          <P>cp277
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: Finland</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-fi
+ *      <TD>EBCDIC: Finland</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-fi
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp278
+ *      <TD>
+ *          <P>cp278
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: Sweden</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-se
+ *      <TD>EBCDIC: Sweden</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-se
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp278
+ *      <TD>
+ *          <P>cp278
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: Italy</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-it
+ *      <TD>EBCDIC: Italy</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-it
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp280
+ *      <TD>
+ *          <P>cp280
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: Spain, Latin America</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-es
+ *      <TD>EBCDIC: Spain, Latin America</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-es
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp284
+ *      <TD>
+ *          <P>cp284
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: Great Britain</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-gb
+ *      <TD>EBCDIC: Great Britain</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-gb
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp285
+ *      <TD>
+ *          <P>cp285
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: France</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-fr
+ *      <TD>EBCDIC: France</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-fr
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp297
+ *      <TD>
+ *          <P>cp297
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: Arabic</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-ar1
+ *      <TD>EBCDIC: Arabic</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-ar1
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp420
+ *      <TD>
+ *          <P>cp420
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: Hebrew</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-he
+ *      <TD>EBCDIC: Hebrew</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-he
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp424
+ *      <TD>
+ *          <P>cp424
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: Switzerland</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-ch
+ *      <TD>EBCDIC: Switzerland</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-ch
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp500
+ *      <TD>
+ *          <P>cp500
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: Roece</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-roece
+ *      <TD>EBCDIC: Roece</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-roece
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp870
+ *      <TD>
+ *          <P>cp870
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: Yogoslavia</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-yu
+ *      <TD>EBCDIC: Yogoslavia</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-yu
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp870
+ *      <TD>
+ *          <P>cp870
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: Iceland</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-is
+ *      <TD>EBCDIC: Iceland</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-is
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp871
+ *      <TD>
+ *          <P>cp871
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">EBCDIC: Urdu</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">ebcdic-cp-ar2
+ *      <TD>EBCDIC: Urdu</TD>
+ *      <TD>
+ *          <P>ebcdic-cp-ar2
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">IANA
+ *      <TD>
+ *          <P>IANA
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">cp918
+ *      <TD>
+ *          <P>cp918
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">Chinese for PRC, mixed 1/2 byte</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">gb2312
+ *      <TD>Chinese for PRC, mixed 1/2 byte</TD>
+ *      <TD>
+ *          <P>gb2312
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">MIME
+ *      <TD>
+ *          <P>MIME
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">GB2312
+ *      <TD>
+ *          <P>GB2312
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">Extended Unix Code, packed for Japanese</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">euc-jp
+ *      <TD>Extended Unix Code, packed for Japanese</TD>
+ *      <TD>
+ *          <P>euc-jp
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">MIME
+ *      <TD>
+ *          <P>MIME
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">eucjis
+ *      <TD>
+ *          <P>eucjis
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">Japanese: iso-2022-jp</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">iso-2020-jp
+ *      <TD>Japanese: iso-2022-jp</TD>
+ *      <TD>
+ *          <P>iso-2020-jp
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">MIME
+ *      <TD>
+ *          <P>MIME
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">JIS
+ *      <TD>
+ *          <P>JIS
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">Japanese: Shift JIS</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">Shift_JIS
+ *      <TD>Japanese: Shift JIS</TD>
+ *      <TD>
+ *          <P>Shift_JIS
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">MIME
+ *      <TD>
+ *          <P>MIME
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">SJIS
+ *      <TD>
+ *          <P>SJIS
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">Chinese: Big5</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">Big5
+ *      <TD>Chinese: Big5</TD>
+ *      <TD>
+ *          <P>Big5
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">MIME
+ *      <TD>
+ *          <P>MIME
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">Big5
+ *      <TD>
+ *          <P>Big5
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">Extended Unix Code, packed for Korean</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">euc-kr
+ *      <TD>Extended Unix Code, packed for Korean</TD>
+ *      <TD>
+ *          <P>euc-kr
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">MIME
+ *      <TD>
+ *          <P>MIME
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">iso2022kr
+ *      <TD>
+ *          <P>iso2022kr
  *      </TD>
  *  </TR>
  *  <TR>
- *      <TD WIDTH="33%">Cyrillic</TD>
- *      <TD WIDTH="15%">
- *          <P ALIGN="CENTER">koi8-r
+ *      <TD>Cyrillic</TD>
+ *      <TD>
+ *          <P>koi8-r
  *      </TD>
- *      <TD WIDTH="12%">
- *          <P ALIGN="CENTER">MIME
+ *      <TD>
+ *          <P>MIME
  *      </TD>
- *      <TD WIDTH="31%">
- *          <P ALIGN="CENTER">koi8-r
+ *      <TD>
+ *          <P>koi8-r
  *      </TD>
  *  </TR>
  * </TABLE>
  *
  * @author TAMURA Kent &lt;kent@trl.ibm.co.jp&gt;
+ *
+ * @deprecated Unused. Will be removed in Tomcat 9.
  */
+@Deprecated
 public class MIME2Java {
 
-    private static Hashtable<String,String> s_enchash;
-    private static Hashtable<String,String> s_revhash;
+    private static final Map<String,String> s_enchash;
+    private static final Map<String,String> s_revhash;
 
     static {
-        s_enchash = new Hashtable<String,String>();
+        s_enchash = new HashMap<>();
         //    <preferred MIME name>, <Java encoding name>
         s_enchash.put("UTF-8", "UTF8");
         s_enchash.put("US-ASCII",        "8859_1");    // ?
@@ -516,10 +520,10 @@ public class MIME2Java {
         s_enchash.put("EBCDIC-CP-IS",    "CP871");
         s_enchash.put("EBCDIC-CP-AR2",   "CP918");
 
-                                                // j:CNS11643 -> EUC-TW?
-                                                // ISO-2022-CN? ISO-2022-CN-EXT?
+        // j:CNS11643 -> EUC-TW?
+        // ISO-2022-CN? ISO-2022-CN-EXT?
 
-        s_revhash = new Hashtable<String,String>();
+        s_revhash = new HashMap<>();
         //    <Java encoding name>, <preferred MIME name>
         s_revhash.put("UTF8", "UTF-8");
         //s_revhash.put("8859_1", "US-ASCII");    // ?
