@@ -39,6 +39,7 @@ import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.http.MimeHeaders;
 import org.apache.tomcat.util.http.parser.Host;
 import org.apache.tomcat.util.net.ApplicationBufferHandler;
+import org.apache.tomcat.util.net.WriteBuffer;
 import org.apache.tomcat.util.res.StringManager;
 
 public class Stream extends AbstractStream implements HeaderEmitter {
@@ -666,10 +667,12 @@ public class Stream extends AbstractStream implements HeaderEmitter {
     }
 
 
-    class StreamOutputBuffer implements HttpOutputBuffer {
+    class StreamOutputBuffer implements HttpOutputBuffer, WriteBuffer.Sink {
 
         private final ByteBuffer buffer = ByteBuffer.allocate(8 * 1024);
+        private final WriteBuffer writeBuffer = new WriteBuffer(32 * 1024);
         private volatile long written = 0;
+        private volatile int streamReservation = 0;
         private volatile boolean closed = false;
         private volatile boolean endOfStreamSent = false;
         private volatile boolean writeInterest = false;
@@ -690,6 +693,9 @@ public class Stream extends AbstractStream implements HeaderEmitter {
                 throw new IllegalStateException(
                         sm.getString("stream.closed", getConnectionId(), getIdentifier()));
             }
+            if (!coyoteResponse.isCommitted()) {
+                coyoteResponse.sendHeaders();
+            }
             int len = chunk.getLength();
             int offset = 0;
             while (len > 0) {
@@ -715,31 +721,61 @@ public class Stream extends AbstractStream implements HeaderEmitter {
                 throw new IllegalStateException(
                         sm.getString("stream.closed", getConnectionId(), getIdentifier()));
             }
-            if (!coyoteResponse.isCommitted()) {
-                coyoteResponse.sendHeaders();
-            }
-            int chunkLimit = chunk.limit();
-            int offset = 0;
-            while (chunk.remaining() > 0) {
-                int thisTime = Math.min(buffer.remaining(), chunk.remaining());
-                chunk.limit(chunk.position() + thisTime);
-                buffer.put(chunk);
-                chunk.limit(chunkLimit);
-                offset += thisTime;
-                if (chunk.remaining() > 0 && !buffer.hasRemaining()) {
-                    // Only flush if we have more data to write and the buffer
-                    // is full
-                    if (flush(true, coyoteResponse.getWriteListener() == null)) {
-                        break;
+            int totalThisTime = 0;
+            if (writeBuffer.isEmpty()) {
+                int chunkLimit = chunk.limit();
+                while (chunk.remaining() > 0) {
+                    int thisTime = Math.min(buffer.remaining(), chunk.remaining());
+                    chunk.limit(chunk.position() + thisTime);
+                    buffer.put(chunk);
+                    chunk.limit(chunkLimit);
+                    totalThisTime += thisTime;
+                    if (chunk.remaining() > 0 && !buffer.hasRemaining()) {
+                        // Only flush if we have more data to write and the buffer
+                        // is full
+                        if (flush(true, coyoteResponse.getWriteListener() == null)) {
+                            totalThisTime = chunk.remaining();
+                            writeBuffer.add(chunk);
+                            break;
+                        }
                     }
                 }
+            } else {
+                totalThisTime = chunk.remaining();
+                writeBuffer.add(chunk);
             }
-            written += offset;
-            return offset;
+            written += totalThisTime;
+            return totalThisTime;
         }
 
         public synchronized boolean flush(boolean block) throws IOException {
-            return flush(false, block);
+            /*
+             * Need to ensure that there is exactly one call to flush even when
+             * there is no data to write.
+             * Too few calls (i.e. zero) and the end of stream message is not
+             * sent for a completed asynchronous write.
+             * Too many calls and the end of stream message is sent too soon and
+             * trailer headers are not sent.
+             */
+            boolean dataLeft = buffer.position() > 0;
+            boolean flushed = false;
+
+            if (dataLeft) {
+                dataLeft = flush(false, block);
+                flushed = true;
+            }
+
+            if (!dataLeft) {
+                if (writeBuffer.isEmpty()) {
+                    if (!flushed) {
+                        dataLeft = flush(false, block);
+                    }
+                } else {
+                    dataLeft = writeBuffer.write(this, block);
+                }
+            }
+
+            return dataLeft;
         }
 
         private synchronized boolean flush(boolean writeInProgress, boolean block)
@@ -761,15 +797,26 @@ public class Stream extends AbstractStream implements HeaderEmitter {
             buffer.flip();
             int left = buffer.remaining();
             while (left > 0) {
-                int streamReservation  = reserveWindowSize(left, block);
                 if (streamReservation == 0) {
-                    // Must be non-blocking
-                    buffer.compact();
-                    return true;
+                    streamReservation  = reserveWindowSize(left, block);
+                    if (streamReservation == 0) {
+                        // Must be non-blocking. Unwritten non-blocking data
+                        // must put in writeBuffer else isReady() logic breaks
+                        writeBuffer.add(buffer);
+                        buffer.clear();
+                        return true;
+                    }
                 }
                 while (streamReservation > 0) {
                     int connectionReservation =
-                                handler.reserveWindowSize(Stream.this, streamReservation);
+                                handler.reserveWindowSize(Stream.this, streamReservation, block);
+                    if (connectionReservation == 0) {
+                        // Must be non-blocking. Unwritten non-blocking data
+                        // must put in writeBuffer else isReady() logic breaks
+                        writeBuffer.add(buffer);
+                        buffer.clear();
+                        return true;
+                    }
                     // Do the write
                     handler.writeBody(Stream.this, buffer, connectionReservation,
                             !writeInProgress && closed && left == connectionReservation);
@@ -782,7 +829,7 @@ public class Stream extends AbstractStream implements HeaderEmitter {
         }
 
         synchronized boolean isReady() {
-            if (getWindowSize() > 0 && handler.getWindowSize() > 0) {
+            if (getWindowSize() > 0 && handler.getWindowSize() > 0 && writeBuffer.isEmpty()) {
                 return true;
             } else {
                 writeInterest = true;
@@ -824,7 +871,31 @@ public class Stream extends AbstractStream implements HeaderEmitter {
 
         @Override
         public void flush() throws IOException {
-            flush(true);
+            /*
+             * This method should only be called during blocking I/O. All the
+             * Servlet API calls that end up here are illegal during
+             * non-blocking I/O. Servlet 5.4.
+             * However, the wording Servlet specification states that the
+             * behaviour is undefined so we do the best we can which is to
+             * perform a flush using blocking I/O or non-blocking I/O based
+             * depending which is currently in use.
+             */
+            flush(getCoyoteResponse().getWriteListener() == null);
+        }
+
+        @Override
+        public synchronized boolean writeFromBuffer(ByteBuffer src, boolean blocking) throws IOException {
+            int chunkLimit = src.limit();
+            while (src.remaining() > 0) {
+                int thisTime = Math.min(buffer.remaining(), src.remaining());
+                src.limit(src.position() + thisTime);
+                buffer.put(src);
+                src.limit(chunkLimit);
+                if (flush(false, blocking)) {
+                    return true;
+                }
+            }
+            return false;
         }
     }
 
