@@ -17,10 +17,8 @@
 
 package org.apache.jasper.compiler;
 
-import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.util.HashMap;
-import java.util.Iterator;
 import java.util.Map.Entry;
 import java.util.Vector;
 
@@ -37,8 +35,8 @@ import org.apache.jasper.JasperException;
 import org.apache.jasper.JspCompilationContext;
 import org.apache.jasper.runtime.JspSourceDependent;
 import org.apache.jasper.servlet.JspServletWrapper;
+import org.apache.tomcat.Jar;
 import org.apache.tomcat.util.descriptor.tld.TldResourcePath;
-import org.apache.tomcat.util.scan.Jar;
 
 /**
  * 1. Processes and extracts the directive info in a tag file. 2. Compiles and
@@ -483,6 +481,8 @@ class TagFileProcessor {
      * @param tagLibInfo
      *            the TagLibraryInfo object associated with this TagInfo
      * @return a TagInfo object assembled from the directives in the tag file.
+     *
+     * @throws JasperException If an error occurs during parsing
      */
     @SuppressWarnings("null") // page can't be null
     public static TagInfo parseTagFileDirectives(ParserController pc,
@@ -495,8 +495,6 @@ class TagFileProcessor {
         Node.Nodes page = null;
         try {
             page = pc.parseTagFileDirectives(path, jar);
-        } catch (FileNotFoundException e) {
-            err.jspError("jsp.error.file.not.found", path);
         } catch (IOException e) {
             err.jspError("jsp.error.file.not.found", path);
         }
@@ -594,12 +592,10 @@ class TagFileProcessor {
                     // Dependent list. The only reliable dependency information
                     // can only be obtained from the tag instance.
                     try {
-                        Object tagIns = tagClazz.newInstance();
+                        Object tagIns = tagClazz.getConstructor().newInstance();
                         if (tagIns instanceof JspSourceDependent) {
-                            Iterator<Entry<String,Long>> iter = ((JspSourceDependent)
-                                    tagIns).getDependants().entrySet().iterator();
-                            while (iter.hasNext()) {
-                                Entry<String,Long> entry = iter.next();
+                            for (Entry<String, Long> entry : ((JspSourceDependent)
+                                    tagIns).getDependants().entrySet()) {
                                 parentPageInfo.addDependant(entry.getKey(),
                                         entry.getValue());
                             }
@@ -660,16 +656,14 @@ class TagFileProcessor {
                                                   Long.valueOf(jar.getLastModified(tagFilePath.substring(1))));
                         } else {
                             pageInfo.addDependant(tagFilePath,
-                                                  compiler.getCompilationContext().getLastModified(
-                                                                                                   tagFilePath));
+                                                  compiler.getCompilationContext().getLastModified(tagFilePath));
                         }
                     } catch (IOException ioe) {
                         throw new JasperException(ioe);
                     }
                 } else {
                     pageInfo.addDependant(tagFilePath,
-                            compiler.getCompilationContext().getLastModified(
-                                    tagFilePath));
+                            compiler.getCompilationContext().getLastModified(tagFilePath));
                 }
                 Class<?> c = loadTagFile(compiler, tagFilePath, n.getTagInfo(),
                         pageInfo);
@@ -684,6 +678,11 @@ class TagFileProcessor {
      * tag files used in a JSP files. The directives in the tag files are
      * assumed to have been processed and encapsulated as TagFileInfo in the
      * CustomTag nodes.
+     *
+     * @param compiler Compiler to use to compile tag files
+     * @param page     The page from to scan for tag files to compile
+     *
+     * @throws JasperException If an error occurs during the scan or compilation
      */
     public void loadTagFiles(Compiler compiler, Node.Nodes page)
             throws JasperException {
@@ -700,9 +699,7 @@ class TagFileProcessor {
      *            If non-null, remove only the class file with with this name.
      */
     public void removeProtoTypeFiles(String classFileName) {
-        Iterator<Compiler> iter = tempVector.iterator();
-        while (iter.hasNext()) {
-            Compiler c = iter.next();
+        for (Compiler c : tempVector) {
             if (classFileName == null) {
                 c.removeGeneratedClassFiles();
             } else if (classFileName.equals(c.getCompilationContext()
