@@ -14,7 +14,6 @@
  *  See the License for the specific language governing permissions and
  *  limitations under the License.
  */
-
 package org.apache.catalina.loader;
 
 
@@ -25,9 +24,6 @@ import java.io.InputStream;
 import java.util.Arrays;
 import java.util.HashSet;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertTrue;
-
 import org.junit.Assert;
 import org.junit.Test;
 
@@ -145,19 +141,19 @@ public class TestVirtualContext extends TomcatBaseTest {
             getUrl(
                 "http://localhost:" + getPort() +
                     "/test/classpathGetResources.jsp?path=rsrc/").toString();
-        assertTrue(
+        Assert.assertTrue(
             allUrls,
             allUrls.indexOf("/test/webapp-virtual-webapp/src/main/webapp/WEB-INF/classes/rsrc") > 0);
-        assertTrue(
+        Assert.assertTrue(
             allUrls,
             allUrls.indexOf("/test/webapp-virtual-webapp/src/main/webapp2/WEB-INF/classes/rsrc") > 0);
-        assertTrue(
+        Assert.assertTrue(
             allUrls,
             allUrls.indexOf("/test/webapp-virtual-webapp/src/main/webapp/WEB-INF/lib/rsrc.jar!/rsrc") > 0);
-        assertTrue(
+        Assert.assertTrue(
             allUrls,
             allUrls.indexOf("/test/webapp-virtual-webapp/target/classes/rsrc") > 0);
-        assertTrue(
+        Assert.assertTrue(
             allUrls,
             allUrls.indexOf("/test/webapp-virtual-library/target/WEB-INF/classes/rsrc") > 0);
 
@@ -170,7 +166,7 @@ public class TestVirtualContext extends TomcatBaseTest {
             getUrl(
                 "http://localhost:" + getPort() +
                     "/test/classpathGetResources.jsp?path=rsrc2/").toString();
-        assertTrue(
+        Assert.assertTrue(
             allRsrsc2ClasspathUrls,
             allRsrsc2ClasspathUrls.indexOf("/test/webapp-virtual-webapp/src/main/webapp2/WEB-INF/classes/rsrc2") > 0);
 
@@ -213,13 +209,13 @@ public class TestVirtualContext extends TomcatBaseTest {
             getUrl(
                 "http://localhost:" + getPort() +
                     "/test/contextGetResourcePaths.jsp?path=/rsrc/").toString();
-        assertTrue(
+        Assert.assertTrue(
             allRsrcPaths,
             allRsrcPaths.indexOf("/rsrc/resourceF.properties") > 0);
-        assertTrue(
+        Assert.assertTrue(
             allRsrcPaths,
             allRsrcPaths.indexOf("/rsrc/resourceE.properties") > 0);
-        assertTrue(
+        Assert.assertTrue(
             allRsrcPaths,
             allRsrcPaths.indexOf("/rsrc/resourceH.properties") > 0);
 
@@ -232,7 +228,7 @@ public class TestVirtualContext extends TomcatBaseTest {
             getUrl(
                 "http://localhost:" + getPort() +
                     "/test/contextGetResourcePaths.jsp?path=/rsrc2/").toString();
-        assertTrue(
+        Assert.assertTrue(
             allRsrc2Paths,
             allRsrc2Paths.indexOf("/rsrc2/resourceJ.properties") > 0);
 
@@ -266,15 +262,13 @@ public class TestVirtualContext extends TomcatBaseTest {
             new File(additionWebInfClasses,
                 MyAnnotatedServlet.class.getPackage().getName().replace('.', '/'));
         Assert.assertTrue(targetPackageForAnnotatedClass.mkdirs());
-        InputStream annotatedServletClassInputStream =
-            this.getClass().getResourceAsStream(
+        try (InputStream annotatedServletClassInputStream = this.getClass().getResourceAsStream(
                 MyAnnotatedServlet.class.getSimpleName() + ".class");
-        FileOutputStream annotatedServletClassOutputStream =
-            new FileOutputStream(new File(targetPackageForAnnotatedClass,
-                MyAnnotatedServlet.class.getSimpleName() + ".class"));
-        IOUtils.copy(annotatedServletClassInputStream, annotatedServletClassOutputStream);
-        annotatedServletClassInputStream.close();
-        annotatedServletClassOutputStream.close();
+                FileOutputStream annotatedServletClassOutputStream = new FileOutputStream(new File(
+                        targetPackageForAnnotatedClass, MyAnnotatedServlet.class.getSimpleName()
+                                + ".class"));) {
+            IOUtils.copy(annotatedServletClassInputStream, annotatedServletClassOutputStream);
+        }
 
         ctx.setResources(new StandardRoot(ctx));
         File f1 = new File("test/webapp-virtual-webapp/target/classes");
@@ -331,11 +325,11 @@ public class TestVirtualContext extends TomcatBaseTest {
         int sc = getUrl("http://localhost:" + getPort() + pageUrl, res, 10000,
                 null, null);
 
-        assertEquals(expectedStatus, sc);
+        Assert.assertEquals(expectedStatus, sc);
 
         if (expectedStatus == 200) {
             String result = res.toString();
-            assertTrue(result, result.indexOf(expectedBody) >= 0);
+            Assert.assertTrue(result, result.indexOf(expectedBody) >= 0);
         }
     }
 }
