@@ -19,18 +19,25 @@ package org.apache.catalina.nonblocking;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.io.Serializable;
 import java.net.HttpURLConnection;
 import java.net.Socket;
 import java.net.URL;
 import java.nio.charset.StandardCharsets;
+import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
+import java.util.concurrent.CountDownLatch;
+import java.util.concurrent.TimeUnit;
 
 import javax.net.SocketFactory;
 import javax.servlet.AsyncContext;
 import javax.servlet.AsyncEvent;
 import javax.servlet.AsyncListener;
+import javax.servlet.DispatcherType;
 import javax.servlet.ReadListener;
 import javax.servlet.ServletException;
 import javax.servlet.ServletInputStream;
@@ -41,10 +48,10 @@ import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
 import org.junit.Assert;
+import org.junit.Ignore;
 import org.junit.Test;
 
 import org.apache.catalina.Context;
-import org.apache.catalina.core.StandardContext;
 import org.apache.catalina.startup.BytesStreamer;
 import org.apache.catalina.startup.TesterServlet;
 import org.apache.catalina.startup.Tomcat;
@@ -93,9 +100,8 @@ public class TestNonBlockingAPI extends TomcatBaseTest {
     private void doTestNonBlockingRead(boolean ignoreIsReady) throws Exception {
         Tomcat tomcat = getTomcatInstance();
 
-        // Must have a real docBase - just use temp
-        StandardContext ctx = (StandardContext) tomcat.addContext("",
-                System.getProperty("java.io.tmpdir"));
+        // No file system docBase required
+        Context ctx = tomcat.addContext("", null);
 
         NBReadServlet servlet = new NBReadServlet(ignoreIsReady);
         String servletName = NBReadServlet.class.getName();
@@ -769,4 +775,224 @@ public class TestNonBlockingAPI extends TomcatBaseTest {
         }
         return rc;
     }
+
+
+    @Ignore
+    @Test
+    public void testDelayedNBWrite() throws Exception {
+        Tomcat tomcat = getTomcatInstance();
+
+        Context ctx = tomcat.addContext("", null);
+        CountDownLatch latch1 = new CountDownLatch(1);
+        DelayedNBWriteServlet servlet = new DelayedNBWriteServlet(latch1);
+        String servletName = DelayedNBWriteServlet.class.getName();
+        Tomcat.addServlet(ctx, servletName, servlet);
+        ctx.addServletMappingDecoded("/", servletName);
+
+        tomcat.start();
+
+        CountDownLatch latch2 = new CountDownLatch(2);
+        List<Throwable> exceptions = new ArrayList<>();
+
+        Thread t = new Thread(
+                new RequestExecutor("http://localhost:" + getPort() + "/", latch2, exceptions));
+        t.start();
+
+        latch1.await(3000, TimeUnit.MILLISECONDS);
+
+        Thread t1 = new Thread(new RequestExecutor(
+                "http://localhost:" + getPort() + "/?notify=true", latch2, exceptions));
+        t1.start();
+
+        latch2.await(3000, TimeUnit.MILLISECONDS);
+
+        if (exceptions.size() > 0) {
+            Assert.fail();
+        }
+    }
+
+    private static final class RequestExecutor implements Runnable {
+        private final String url;
+        private final CountDownLatch latch;
+        private final List<Throwable> exceptions;
+
+        public RequestExecutor(String url, CountDownLatch latch, List<Throwable> exceptions) {
+            this.url = url;
+            this.latch = latch;
+            this.exceptions = exceptions;
+        }
+
+        @Override
+        public void run() {
+            try {
+                ByteChunk result = new ByteChunk();
+                int rc = getUrl(url, result, null);
+                Assert.assertTrue(rc == HttpServletResponse.SC_OK);
+                Assert.assertTrue(result.toString().contains("OK"));
+            } catch (Throwable e) {
+                e.printStackTrace();
+                exceptions.add(e);
+            } finally {
+                latch.countDown();
+            }
+        }
+
+    }
+
+    @WebServlet(asyncSupported = true)
+    private static final class DelayedNBWriteServlet extends TesterServlet {
+        private static final long serialVersionUID = 1L;
+        private final Set<Emitter> emitters = new HashSet<>();
+        private final transient CountDownLatch latch;
+
+        public DelayedNBWriteServlet(CountDownLatch latch) {
+            this.latch = latch;
+        }
+
+        @Override
+        protected void doGet(HttpServletRequest request, HttpServletResponse response)
+                throws ServletException, IOException {
+            boolean notify = Boolean.parseBoolean(request.getParameter("notify"));
+            AsyncContext ctx = request.startAsync();
+            ctx.setTimeout(1000);
+            if (!notify) {
+                emitters.add(new Emitter(ctx));
+                latch.countDown();
+            } else {
+                for (Emitter e : emitters) {
+                    e.emit();
+                }
+                response.getOutputStream().println("OK");
+                response.getOutputStream().flush();
+                ctx.complete();
+            }
+        }
+
+    }
+
+    private static final class Emitter implements Serializable {
+
+        private static final long serialVersionUID = 1L;
+
+        private final transient AsyncContext ctx;
+
+        Emitter(AsyncContext ctx) {
+            this.ctx = ctx;
+        }
+
+        void emit() throws IOException {
+            ctx.getResponse().getOutputStream().setWriteListener(new WriteListener() {
+                private boolean written = false;
+
+                @Override
+                public void onWritePossible() throws IOException {
+                    ServletOutputStream out = ctx.getResponse().getOutputStream();
+                    if (out.isReady() && !written) {
+                        out.println("OK");
+                        written = true;
+                    }
+                    if (out.isReady() && written) {
+                        out.flush();
+                        if (out.isReady()) {
+                            ctx.complete();
+                        }
+                    }
+                }
+
+                @Override
+                public void onError(Throwable t) {
+                    t.printStackTrace();
+                }
+
+            });
+        }
+    }
+
+
+    /*
+     * https://bz.apache.org/bugzilla/show_bug.cgi?id=61932
+     */
+    @Test
+    public void testNonBlockingReadWithDispatch() throws Exception {
+        Tomcat tomcat = getTomcatInstance();
+
+        // No file system docBase required
+        Context ctx = tomcat.addContext("", null);
+
+        NBReadWithDispatchServlet servlet = new NBReadWithDispatchServlet();
+        String servletName = NBReadWithDispatchServlet.class.getName();
+        Tomcat.addServlet(ctx, servletName, servlet);
+        ctx.addServletMappingDecoded("/", servletName);
+
+        tomcat.start();
+
+        Map<String, List<String>> resHeaders = new HashMap<>();
+        int rc = postUrl(true, new DataWriter(500), "http://localhost:" +
+                getPort() + "/", new ByteChunk(), resHeaders, null);
+
+        Assert.assertEquals(HttpServletResponse.SC_OK, rc);
+    }
+
+
+    @WebServlet(asyncSupported = true)
+    private final class NBReadWithDispatchServlet extends TesterServlet {
+
+        private static final long serialVersionUID = 1L;
+
+        @Override
+        protected void doPost(HttpServletRequest req, HttpServletResponse resp)
+                throws ServletException, IOException {
+
+            final CountDownLatch latch = new CountDownLatch(1);
+
+            // Dispatch to "/error" will end up here
+            if (req.getDispatcherType().equals(DispatcherType.ASYNC)) {
+                // Return without writing anything. This will generate the
+                // expected 200 response.
+                return;
+            }
+
+            final AsyncContext asyncCtx = req.startAsync();
+            final ServletInputStream is = req.getInputStream();
+            is.setReadListener(new ReadListener() {
+
+                @Override
+                public void onDataAvailable() {
+
+                    try {
+                        byte buffer[] = new byte[1 * 1024];
+                        while (is.isReady() && !is.isFinished()) {
+                            is.read(buffer);
+                        }
+
+                    } catch (IOException ex) {
+                        ex.printStackTrace();
+                    }
+                }
+
+                @Override
+                public void onAllDataRead() {
+                    latch.countDown();
+                }
+
+                @Override
+                public void onError(Throwable t) {
+                }
+            });
+
+            Thread t = new Thread() {
+
+                @Override
+                public void run() {
+                    try {
+                        latch.await();
+                    } catch (InterruptedException e) {
+                        e.printStackTrace();
+                    }
+                    asyncCtx.dispatch("/error");
+                }
+            };
+            t.start();
+        }
+    }
 }
