@@ -900,6 +900,7 @@ public abstract class Http2TestBase extends TomcatBaseTest {
         private ConnectionSettingsRemote remoteSettings = new ConnectionSettingsRemote("-1");
         private boolean traceBody = false;
         private ByteBuffer bodyBuffer = null;
+        private long bytesRead;
 
         public void setTraceBody(boolean traceBody) {
             this.traceBody = traceBody;
@@ -915,6 +916,7 @@ public abstract class Http2TestBase extends TomcatBaseTest {
         @Override
         public ByteBuffer startRequestBodyFrame(int streamId, int payloadSize) {
             lastStreamId = Integer.toString(streamId);
+            bytesRead += payloadSize;
             if (traceBody) {
                 bodyBuffer = ByteBuffer.allocate(payloadSize);
                 return bodyBuffer;
@@ -1078,6 +1080,7 @@ public abstract class Http2TestBase extends TomcatBaseTest {
 
         public void clearTrace() {
             trace = new StringBuffer();
+            bytesRead = 0;
         }
 
 
@@ -1089,6 +1092,11 @@ public abstract class Http2TestBase extends TomcatBaseTest {
         public int getMaxFrameSize() {
             return remoteSettings.getMaxFrameSize();
         }
+
+
+        public long getBytesRead() {
+            return bytesRead;
+        }
     }
 
 
@@ -1111,6 +1119,8 @@ public abstract class Http2TestBase extends TomcatBaseTest {
 
         private static final long serialVersionUID = 1L;
 
+        public static final int CONTENT_LENGTH = 8192;
+
         @Override
         protected void doGet(HttpServletRequest req, HttpServletResponse resp)
                 throws ServletException, IOException {
